@@ -8,9 +8,11 @@ import (
 	"github.com/getkin/kin-openapi/jsoninfo"
 )
 
-// Components is specified by OpenAPI/Swagger standard version 3.0.
+// Components is specified by OpenAPI/Swagger standard version 3.
+// See https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.0.3.md#componentsObject
 type Components struct {
 	ExtensionProps
+
 	Schemas         Schemas         `json:"schemas,omitempty" yaml:"schemas,omitempty"`
 	Parameters      ParametersMap   `json:"parameters,omitempty" yaml:"parameters,omitempty"`
 	Headers         Headers         `json:"headers,omitempty" yaml:"headers,omitempty"`
@@ -26,14 +28,17 @@ func NewComponents() Components {
 	return Components{}
 }
 
+// MarshalJSON returns the JSON encoding of Components.
 func (components *Components) MarshalJSON() ([]byte, error) {
 	return jsoninfo.MarshalStrictStruct(components)
 }
 
+// UnmarshalJSON sets Components to a copy of data.
 func (components *Components) UnmarshalJSON(data []byte) error {
 	return jsoninfo.UnmarshalStrictStruct(data, components)
 }
 
+// Validate returns an error if Components does not comply with the OpenAPI spec.
 func (components *Components) Validate(ctx context.Context) (err error) {
 	for k, v := range components.Schemas {
 		if err = ValidateIdentifier(k); err != nil {
@@ -89,6 +94,33 @@ func (components *Components) Validate(ctx context.Context) (err error) {
 		}
 	}
 
+	for k, v := range components.Examples {
+		if err = ValidateIdentifier(k); err != nil {
+			return
+		}
+		if err = v.Validate(ctx); err != nil {
+			return
+		}
+	}
+
+	for k, v := range components.Links {
+		if err = ValidateIdentifier(k); err != nil {
+			return
+		}
+		if err = v.Validate(ctx); err != nil {
+			return
+		}
+	}
+
+	for k, v := range components.Callbacks {
+		if err = ValidateIdentifier(k); err != nil {
+			return
+		}
+		if err = v.Validate(ctx); err != nil {
+			return
+		}
+	}
+
 	return
 }
 
