@@ -275,6 +275,12 @@ var schemaExamples = []schemaExample{
 			"dd7d8481-81a3-407f-95f0-a2f1cb382a4b",
 			"dcba3901-2fba-48c1-9db2-00422055804e",
 			"ace8e3be-c254-4c10-8859-1401d9a9d52a",
+			"DD7D8481-81A3-407F-95F0-A2F1CB382A4B",
+			"DCBA3901-2FBA-48C1-9DB2-00422055804E",
+			"ACE8E3BE-C254-4C10-8859-1401D9A9D52A",
+			"dd7D8481-81A3-407f-95F0-A2F1CB382A4B",
+			"DCBA3901-2FBA-48C1-9db2-00422055804e",
+			"ACE8E3BE-c254-4C10-8859-1401D9A9D52A",
 		},
 		AllInvalid: []interface{}{
 			nil,
@@ -282,6 +288,13 @@ var schemaExamples = []schemaExample{
 			"4cf3i040-ea14-4daa-b0b5-ea9329473519",
 			"aaf85740-7e27-4b4f-b4554-a03a43b1f5e3",
 			"56f5bff4-z4b6-48e6-a10d-b6cf66a83b04",
+			"G39840B1-D0EF-446D-E555-48FCCA50A90A",
+			"4CF3I040-EA14-4DAA-B0B5-EA9329473519",
+			"AAF85740-7E27-4B4F-B4554-A03A43B1F5E3",
+			"56F5BFF4-Z4B6-48E6-A10D-B6CF66A83B04",
+			"4CF3I040-EA14-4Daa-B0B5-EA9329473519",
+			"AAf85740-7E27-4B4F-B4554-A03A43b1F5E3",
+			"56F5Bff4-Z4B6-48E6-a10D-B6CF66A83B04",
 		},
 	},
 
