@@ -5,12 +5,14 @@ import (
 	"errors"
 	"fmt"
 
-	"github.com/getkin/kin-openapi/jsoninfo"
 	"github.com/go-openapi/jsonpointer"
+
+	"github.com/getkin/kin-openapi/jsoninfo"
 )
 
 type SecuritySchemes map[string]*SecuritySchemeRef
 
+// JSONLookup implements github.com/go-openapi/jsonpointer#JSONPointable
 func (s SecuritySchemes) JSONLookup(token string) (interface{}, error) {
 	ref, ok := s[token]
 	if ref == nil || ok == false {
@@ -25,6 +27,8 @@ func (s SecuritySchemes) JSONLookup(token string) (interface{}, error) {
 
 var _ jsonpointer.JSONPointable = (*SecuritySchemes)(nil)
 
+// SecurityScheme is specified by OpenAPI/Swagger standard version 3.
+// See https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.0.3.md#securitySchemeObject
 type SecurityScheme struct {
 	ExtensionProps
 
@@ -65,10 +69,12 @@ func NewJWTSecurityScheme() *SecurityScheme {
 	}
 }
 
+// MarshalJSON returns the JSON encoding of SecurityScheme.
 func (ss *SecurityScheme) MarshalJSON() ([]byte, error) {
 	return jsoninfo.MarshalStrictStruct(ss)
 }
 
+// UnmarshalJSON sets SecurityScheme to a copy of data.
 func (ss *SecurityScheme) UnmarshalJSON(data []byte) error {
 	return jsoninfo.UnmarshalStrictStruct(data, ss)
 }
@@ -103,15 +109,16 @@ func (ss *SecurityScheme) WithBearerFormat(value string) *SecurityScheme {
 	return ss
 }
 
-func (value *SecurityScheme) Validate(ctx context.Context) error {
+// Validate returns an error if SecurityScheme does not comply with the OpenAPI spec.
+func (ss *SecurityScheme) Validate(ctx context.Context) error {
 	hasIn := false
 	hasBearerFormat := false
 	hasFlow := false
-	switch value.Type {
+	switch ss.Type {
 	case "apiKey":
 		hasIn = true
 	case "http":
-		scheme := value.Scheme
+		scheme := ss.Scheme
 		switch scheme {
 		case "bearer":
 			hasBearerFormat = true
@@ -122,52 +129,55 @@ func (value *SecurityScheme) Validate(ctx context.Context) error {
 	case "oauth2":
 		hasFlow = true
 	case "openIdConnect":
-		if value.OpenIdConnectUrl == "" {
-			return fmt.Errorf("no OIDC URL found for openIdConnect security scheme %q", value.Name)
+		if ss.OpenIdConnectUrl == "" {
+			return fmt.Errorf("no OIDC URL found for openIdConnect security scheme %q", ss.Name)
 		}
 	default:
-		return fmt.Errorf("security scheme 'type' can't be %q", value.Type)
+		return fmt.Errorf("security scheme 'type' can't be %q", ss.Type)
 	}
 
 	// Validate "in" and "name"
 	if hasIn {
-		switch value.In {
+		switch ss.In {
 		case "query", "header", "cookie":
 		default:
-			return fmt.Errorf("security scheme of type 'apiKey' should have 'in'. It can be 'query', 'header' or 'cookie', not %q", value.In)
+			return fmt.Errorf("security scheme of type 'apiKey' should have 'in'. It can be 'query', 'header' or 'cookie', not %q", ss.In)
 		}
-		if value.Name == "" {
+		if ss.Name == "" {
 			return errors.New("security scheme of type 'apiKey' should have 'name'")
 		}
-	} else if len(value.In) > 0 {
-		return fmt.Errorf("security scheme of type %q can't have 'in'", value.Type)
-	} else if len(value.Name) > 0 {
+	} else if len(ss.In) > 0 {
+		return fmt.Errorf("security scheme of type %q can't have 'in'", ss.Type)
+	} else if len(ss.Name) > 0 {
 		return errors.New("security scheme of type 'apiKey' can't have 'name'")
 	}
 
 	// Validate "format"
 	// "bearerFormat" is an arbitrary string so we only check if the scheme supports it
-	if !hasBearerFormat && len(value.BearerFormat) > 0 {
-		return fmt.Errorf("security scheme of type %q can't have 'bearerFormat'", value.Type)
+	if !hasBearerFormat && len(ss.BearerFormat) > 0 {
+		return fmt.Errorf("security scheme of type %q can't have 'bearerFormat'", ss.Type)
 	}
 
 	// Validate "flow"
 	if hasFlow {
-		flow := value.Flows
+		flow := ss.Flows
 		if flow == nil {
-			return fmt.Errorf("security scheme of type %q should have 'flows'", value.Type)
+			return fmt.Errorf("security scheme of type %q should have 'flows'", ss.Type)
 		}
 		if err := flow.Validate(ctx); err != nil {
 			return fmt.Errorf("security scheme 'flow' is invalid: %v", err)
 		}
-	} else if value.Flows != nil {
-		return fmt.Errorf("security scheme of type %q can't have 'flows'", value.Type)
+	} else if ss.Flows != nil {
+		return fmt.Errorf("security scheme of type %q can't have 'flows'", ss.Type)
 	}
 	return nil
 }
 
+// OAuthFlows is specified by OpenAPI/Swagger standard version 3.
+// See https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.0.3.md#oauthFlowsObject
 type OAuthFlows struct {
 	ExtensionProps
+
 	Implicit          *OAuthFlow `json:"implicit,omitempty" yaml:"implicit,omitempty"`
 	Password          *OAuthFlow `json:"password,omitempty" yaml:"password,omitempty"`
 	ClientCredentials *OAuthFlow `json:"clientCredentials,omitempty" yaml:"clientCredentials,omitempty"`
@@ -183,14 +193,17 @@ const (
 	oAuthFlowAuthorizationCode
 )
 
+// MarshalJSON returns the JSON encoding of OAuthFlows.
 func (flows *OAuthFlows) MarshalJSON() ([]byte, error) {
 	return jsoninfo.MarshalStrictStruct(flows)
 }
 
+// UnmarshalJSON sets OAuthFlows to a copy of data.
 func (flows *OAuthFlows) UnmarshalJSON(data []byte) error {
 	return jsoninfo.UnmarshalStrictStruct(data, flows)
 }
 
+// Validate returns an error if OAuthFlows does not comply with the OpenAPI spec.
 func (flows *OAuthFlows) Validate(ctx context.Context) error {
 	if v := flows.Implicit; v != nil {
 		return v.Validate(ctx, oAuthFlowTypeImplicit)
@@ -207,22 +220,28 @@ func (flows *OAuthFlows) Validate(ctx context.Context) error {
 	return errors.New("no OAuth flow is defined")
 }
 
+// OAuthFlow is specified by OpenAPI/Swagger standard version 3.
+// See https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.0.3.md#oauthFlowObject
 type OAuthFlow struct {
 	ExtensionProps
+
 	AuthorizationURL string            `json:"authorizationUrl,omitempty" yaml:"authorizationUrl,omitempty"`
 	TokenURL         string            `json:"tokenUrl,omitempty" yaml:"tokenUrl,omitempty"`
 	RefreshURL       string            `json:"refreshUrl,omitempty" yaml:"refreshUrl,omitempty"`
 	Scopes           map[string]string `json:"scopes" yaml:"scopes"`
 }
 
+// MarshalJSON returns the JSON encoding of OAuthFlow.
 func (flow *OAuthFlow) MarshalJSON() ([]byte, error) {
 	return jsoninfo.MarshalStrictStruct(flow)
 }
 
+// UnmarshalJSON sets OAuthFlow to a copy of data.
 func (flow *OAuthFlow) UnmarshalJSON(data []byte) error {
 	return jsoninfo.UnmarshalStrictStruct(data, flow)
 }
 
+// Validate returns an error if OAuthFlow does not comply with the OpenAPI spec.
 func (flow *OAuthFlow) Validate(ctx context.Context, typ oAuthFlowType) error {
 	if typ == oAuthFlowAuthorizationCode || typ == oAuthFlowTypeImplicit {
 		if v := flow.AuthorizationURL; v == "" {
